@@ -15,8 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- *  $Id$
  */
 
 #include "config.h"
@@ -28,6 +26,7 @@
 #include <libgames-support/games-card-images.h>
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-pixbuf-utils.h>
+#include <libgames-support/games-sound.h>
 
 #include "conf.h"
 
@@ -49,7 +48,7 @@
 #define MAX_OVERHANG (0.2)
 
 /* The proportion of a slot dedicated to the card (horiz or vert). */
-#ifdef HAVE_MAEMO
+#ifdef HAVE_HILDON
 #define CARD_SLOT_PROP (0.9)
 #else
 #define CARD_SLOT_PROP (0.8)
@@ -61,11 +60,11 @@
 #define STATIC_ASSERT_IMPL(condition, line) STATIC_ASSERT_IMPL2(condition, line)
 #define STATIC_ASSERT_IMPL2(condition, line) typedef int _static_assert_line_##line[(condition) ? 1 : -1]
 
-#ifdef HAVE_MAEMO
+#ifdef HAVE_HILDON 
 #define PIXBUF_DRAWING_LIKELIHOOD(cond) G_UNLIKELY (cond)
 #else
 #define PIXBUF_DRAWING_LIKELIHOOD(cond) (cond)
-#endif /* HAVE_MAEMO */
+#endif /* HAVE_HILDON */
 
 typedef enum {
   CURSOR_DEFAULT,
@@ -196,7 +195,7 @@ static void slot_update_card_images_full (AisleriotBoard *board,
                                           Slot *slot,
                                           int highlight_start_card_id);
 
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
 
 /* These cursors borrowed from EOG */
 /* FIXMEchpe use themeable cursors here! */
@@ -240,11 +239,11 @@ static const char hand_open_mask_bits[] = {
   0x80, 0x7f, 0x00, 0x80, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
 };
 
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 
 /* Cursor */
 
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
 
 static GdkCursor *
 make_cursor (GtkWidget *widget,
@@ -269,18 +268,18 @@ make_cursor (GtkWidget *widget,
   return cursor;
 }
 
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 
 static void
 set_cursor (AisleriotBoard *board,
             CursorType cursor)
 {
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
   AisleriotBoardPrivate *priv = board->priv;
 
   gdk_window_set_cursor (GTK_WIDGET (board)->window,
                          priv->cursor[cursor]);
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 }
 
 /* If we are over a slot, set the cursor to the given cursor,
@@ -290,7 +289,7 @@ set_cursor_by_location (AisleriotBoard *board,
                         int x,
                         int y)
 {
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
   AisleriotBoardPrivate *priv = board->priv;
   Slot *selection_slot = priv->selection_slot;
   int selection_start_card_id = priv->selection_start_card_id;
@@ -329,7 +328,7 @@ set_cursor_by_location (AisleriotBoard *board,
   }
 
   set_cursor (board, cursor);
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 }
 
 /* card drawing functions */
@@ -525,7 +524,7 @@ set_focus (AisleriotBoard *board,
   int top_card_id;
 
   /* Sanitise */
-  top_card_id = slot ? slot->cards->len - 1 : -1;
+  top_card_id = slot ? ((int) slot->cards->len) - 1 : -1;
   card_id = MAX (card_id, top_card_id);
 
   if (priv->focus_slot == slot &&
@@ -936,11 +935,16 @@ drag_begin (AisleriotBoard *board)
   GdkDrawable *drawable;
   GdkWindowAttr attributes;
 
+  if (!priv->selection_slot ||
+      priv->selection_start_card_id < 0) {
+    priv->click_status = STATUS_NONE;
+    return;
+  }
+
   priv->click_status = STATUS_IS_DRAG;
 
   hslot = priv->moving_cards_origin_slot = priv->selection_slot;
   priv->moving_cards_origin_card_id = priv->selection_start_card_id;
-  g_assert (hslot != NULL && priv->moving_cards_origin_card_id >= 0);
 
   num_moving_cards = hslot->cards->len - priv->moving_cards_origin_card_id;
 
@@ -1075,6 +1079,8 @@ drag_begin (AisleriotBoard *board)
 
   slot_update_geometry (board, hslot);
   slot_update_card_images (board, hslot);
+      
+  set_cursor (board, CURSOR_CLOSED);
 }
 
 static void
@@ -1192,8 +1198,10 @@ drop_moving_cards (AisleriotBoard *board,
 
   if (moved) {
     aisleriot_game_end_move (priv->game);
+    games_sound_play ("click");
   } else {
     aisleriot_game_discard_move (priv->game);
+    games_sound_play ("slide");
   }
 
   drag_end (board, moved);
@@ -1378,8 +1386,22 @@ slot_changed_cb (AisleriotGame *game,
   slot_update_geometry (board, slot);
   slot_update_card_images (board, slot);
 
+  if (slot == priv->moving_cards_origin_slot) {
+    /* PANIC! */
+    /* FIXMEchpe */
+  }
   if (slot == priv->selection_slot) {
     set_selection (board, NULL, -1);
+
+    /* If this slot changes while we're in a click cycle, abort the action.
+     * That prevents a problem where the cards that were selected and
+     * about to be dragged have vanished because of autoplay; c.f. bug #449767.
+     * Note: we don't use clear_state() here since we only want to disable
+     * the highlight and revealed card if that particular slot changed, see
+     * the code above. And we don't clear last_clicked_slot/card_id either, so
+     * a double-click will still work.
+     */
+    priv->click_status = STATUS_NONE;
   }
   if (slot == priv->focus_slot) {
     set_focus (board, slot, priv->focus_card_id, priv->show_focus);
@@ -1400,12 +1422,12 @@ aisleriot_board_realize (GtkWidget *widget)
   AisleriotBoardPrivate *priv = board->priv;
   GdkDisplay *display;
   GtkSettings *settings;
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
   gboolean touchscreen_mode;
 #ifdef GDK_WINDOWING_X11
   char *xft_rgba = NULL;
 #endif /* GDK_WINDOWING_X11 */
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 
   GTK_WIDGET_CLASS (aisleriot_board_parent_class)->realize (widget);
 
@@ -1420,28 +1442,28 @@ aisleriot_board_realize (GtkWidget *widget)
   
   priv->slot_gc = gdk_gc_new (widget->window);
 
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
   /* Create cursors */
   priv->cursor[CURSOR_DEFAULT] = gdk_cursor_new_for_display (display, GDK_LEFT_PTR);
   priv->cursor[CURSOR_OPEN] = make_cursor (widget, hand_open_data_bits, hand_open_mask_bits);
   priv->cursor[CURSOR_CLOSED] = make_cursor (widget, hand_closed_data_bits, hand_closed_mask_bits);
   priv->cursor[CURSOR_DROPPABLE] = gdk_cursor_new_for_display (display, GDK_DOUBLE_ARROW); /* FIXMEchpe: better cursor */
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 
    /* Set up the double-click detection. */
   settings = gtk_widget_get_settings (widget);
   /* FIXMEchpe listen to changes! */
   g_object_get (settings,
                 "gtk-double-click-time", &priv->double_click_time,
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
                 "gtk-touchscreen-mode", &touchscreen_mode,
 #ifdef GDK_WINDOWING_X11
                 "gtk-xft-rgba", &xft_rgba,
 #endif /* GDK_WINDOWING_X11 */
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
                 NULL);
 
-#if defined (GDK_WINDOWING_X11) && !defined (HAVE_MAEMO)
+#if defined (GDK_WINDOWING_X11) && !defined (HAVE_HILDON)
   if (xft_rgba != NULL) {
     gboolean antialias_set = TRUE;
     cairo_antialias_t antialias_mode = CAIRO_ANTIALIAS_DEFAULT;
@@ -1471,10 +1493,10 @@ aisleriot_board_realize (GtkWidget *widget)
                                        subpixel_order);
     }
   }
-#endif /* GDK_WINDOWING_X11 && !HAVE_MAEMO */
-#ifndef HAVE_MAEMO
+#endif /* GDK_WINDOWING_X11 && !HAVE_HILDON */
+#ifndef HAVE_HILDON 
   priv->touchscreen_mode = touchscreen_mode != FALSE;
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON */
 
   aisleriot_board_setup_geometry (board);
 }
@@ -1484,7 +1506,7 @@ aisleriot_board_unrealize (GtkWidget *widget)
 {
   AisleriotBoard *board = AISLERIOT_BOARD (widget);
   AisleriotBoardPrivate *priv = board->priv;
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
   guint i;
 #endif
 
@@ -1497,12 +1519,12 @@ aisleriot_board_unrealize (GtkWidget *widget)
   g_object_unref (priv->slot_gc);
   priv->slot_gc = NULL;
 
-#ifndef HAVE_MAEMO
+#ifndef HAVE_HILDON 
   for (i = 0; i < LAST_CURSOR; ++i) {
     gdk_cursor_unref (priv->cursor[i]);
     priv->cursor[i] = NULL;
   }
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_HILDON*/
 
   games_card_images_set_drawable (priv->images, NULL);
 
@@ -1571,6 +1593,13 @@ static gboolean
 aisleriot_board_focus_in (GtkWidget *widget,
                           GdkEventFocus *event)
 {
+  AisleriotBoard *board = AISLERIOT_BOARD (widget);
+  AisleriotBoardPrivate *priv = board->priv;
+
+  /* If we're not showing focus, no need to redraw on focus change */
+  if (!priv->show_focus)
+    return FALSE;
+
   return GTK_WIDGET_CLASS (aisleriot_board_parent_class)->focus_in_event (widget, event);
 }
 
@@ -1579,9 +1608,14 @@ aisleriot_board_focus_out (GtkWidget *widget,
                            GdkEventFocus *event)
 {
   AisleriotBoard *board = AISLERIOT_BOARD (widget);
+  AisleriotBoardPrivate *priv = board->priv;
 
   clear_state (board);
 
+  /* If we're not showing focus, no need to redraw on focus change */
+  if (!priv->show_focus)
+    return FALSE;
+
   return GTK_WIDGET_CLASS (aisleriot_board_parent_class)->focus_out_event (widget, event);
 }
 
@@ -1723,8 +1757,8 @@ aisleriot_board_button_press (GtkWidget *widget,
                                 selection_slot->cards->len);
 
     /* Store the cards, since the move could alter slot->cards! */
+    g_assert (selection_slot->cards->len >= selection_start_card_id);
     n_cards = selection_slot->cards->len - selection_start_card_id;
-    g_assert (n_cards >= 0);
 
     cards = g_alloca (n_cards);
     memcpy (cards,
@@ -1847,6 +1881,7 @@ aisleriot_board_button_release (GtkWidget *widget,
       aisleriot_game_record_move (priv->game, -1, NULL, 0);
       if (aisleriot_game_button_clicked_lambda (priv->game, slot->id)) {
         aisleriot_game_end_move (priv->game);
+	games_sound_play ("click");
       } else {
         aisleriot_game_discard_move (priv->game);
       }
@@ -1900,7 +1935,6 @@ aisleriot_board_motion_notify (GtkWidget *widget,
                                        event->x,
                                        event->y)) {
     drag_begin (board);
-    set_cursor (board, CURSOR_CLOSED);
   } else {
     set_cursor_by_location (board, event->x, event->y);
   }
