@@ -42,15 +42,19 @@
 #include <libgnomeui/gnome-ui-init.h>
 #endif /* HAVE_GNOME */
 
-#ifdef HAVE_MAEMO
+#ifdef HAVE_HILDON
 #include <libosso.h>
+
+#ifdef HAVE_MAEMO_3
 #include <osso-browser-interface.h>
 #include <hildon-widgets/hildon-program.h>
+#else
+#include <hildon/hildon-program.h>
+#endif /* HAVE_MAEMO_3 */
 
 #define SERVICE_NAME "org.gnome.Games.AisleRiot"
 #define HELP_EXT "xhtml"
-
-#endif /* HAVE_MAEMO */
+#endif /* HAVE_HILDON */
 
 #if defined (G_OS_WIN32)
 #include <windows.h>
@@ -59,10 +63,11 @@
 //FIXME On win32 help is created as html with gnome-doc-tool,
 //and put manually in the directory below.
 #define HELPDIR PKGDATADIR "/aisleriot/help"
-#endif
+#endif /* G_OS_WIN32 */
 
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-stock.h>
+#include <libgames-support/games-sound.h>
 
 #include "conf.h"
 #include "game.h"
@@ -77,7 +82,7 @@ typedef struct {
 #ifdef HAVE_GNOME
   GnomeProgram *program;
 #endif
-#ifdef HAVE_MAEMO
+#ifdef HAVE_HILDON
   osso_context_t *osso_context;
   HildonProgram *program;
 #endif
@@ -246,7 +251,7 @@ game_file_to_help_section (const char *game_file)
   return buf;
 }
 
-#if (defined HAVE_MAEMO || defined G_OS_WIN32)
+#if defined (HAVE_HILDON) || defined (G_OS_WIN32)
 
 static void
 help_hook (GtkWindow *parent,
@@ -393,12 +398,12 @@ help_hook (GtkWindow *parent,
     g_free (primary);
     g_error_free (error);
 
-#ifdef HAVE_MAEMO
+#ifdef HAVE_HILDON
   /* Empty title shows up as "<unnamed>" on maemo */
   gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
 #else
   gtk_window_set_title (GTK_WINDOW (dialog), "");
-#endif /* HAVE_MAEMO */
+#endif /* HAVE_HILDON */
 
     gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
@@ -414,7 +419,7 @@ help_hook (GtkWindow *parent,
 #endif /* !HAVE_GNOME */
 }
 
-#endif /* HAVE_MAEMO */
+#endif /* HAVE_HILDON */
 
 #ifdef HAVE_MAEMO
 
@@ -510,7 +515,7 @@ main_prog (void *closure, int argc, char *argv[])
   GError *error = NULL;
   gboolean retval;
 #endif
-#if HAVE_MAEMO
+#ifdef HAVE_MAEMO
   osso_hw_state_t hw_events = {
     TRUE /* shutdown */,
     TRUE /* save unsaved data */,
@@ -556,6 +561,9 @@ main_prog (void *closure, int argc, char *argv[])
 
   add_main_options (option_context, &data);
 
+  games_sound_enable (FALSE);
+  games_sound_add_option_group (option_context);
+
 #ifdef HAVE_GNOME
   data.program = gnome_program_init ("aisleriot", VERSION,
                                      LIBGNOMEUI_MODULE,
@@ -631,7 +639,7 @@ main_prog (void *closure, int argc, char *argv[])
                             G_CALLBACK (die_cb), &data);
 #endif /* HAVE_GNOME */
 
-#ifdef HAVE_MAEMO
+#ifdef HAVE_HILDON
   hildon_program_add_window (data.program, HILDON_WINDOW (data.window));
 
   /* FIXMEchpe sort of strange that maemo doesn't all of this out-of-the-box... */
@@ -685,11 +693,9 @@ cleanup:
 int
 main (int argc, char *argv[])
 {
-  setlocale (LC_ALL, "");
+  char *localedir;
 
-  bindtextdomain (GETTEXT_PACKAGE, games_path_runtime_fix (GNOMELOCALEDIR));
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
+  setlocale (LC_ALL, "");
 
 #if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
   /* If we're going to use gnome-vfs, we need to init threads before
@@ -698,6 +704,12 @@ main (int argc, char *argv[])
   g_thread_init (NULL);
 #endif
 
+  localedir = games_path_runtime_fix (GNOMELOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, localedir);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
+  g_free (localedir);
+
   scm_boot_guile (argc, argv, main_prog, NULL);
 
   return 0;
