@@ -39,11 +39,11 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <gconf/gconf-client.h>
 
 #include <games-stock.h>
 #include <games-scores.h>
 #include <games-scores-dialog.h>
+#include <games-conf.h>
 
 #include "yahtzee.h"
 #include "gyahtzee.h"
@@ -66,6 +66,9 @@ static gboolean ready_to_advance_player;
 #define DIE_SELECTED_PIXMAP  (NUMBER_OF_PIXMAPS-1)
 #define SCORES_CATEGORY (game_type == GAME_KISMET ? "Colors" : NULL)
 
+#define DEFAULT_WIDTH 640
+#define DEFAULT_HEIGHT 480
+
 static char *dicefiles[NUMBER_OF_PIXMAPS] = { PP "gnome-dice-1.svg",
   PP "gnome-dice-2.svg",
   PP "gnome-dice-3.svg",
@@ -93,7 +96,10 @@ static GtkToolItem *diceBox[NUMBER_OF_DICE];
 static GtkWidget *rollLabel;
 static GtkWidget *mbutton;
 static GtkAction *scores_action;
+static GtkAction *undo_action;
 static gchar *game_type_string = NULL;
+static gint   test_computer_play = 0;
+gint NUM_TRIALS = 0;
 
 const static GOptionEntry yahtzee_options[] = {
   {"delay", 'd', 0, G_OPTION_ARG_NONE, &DoDelay,
@@ -106,6 +112,10 @@ const static GOptionEntry yahtzee_options[] = {
    N_("Number of human opponents"), N_("NUMBER")},
   {"game", 'g', 0, G_OPTION_ARG_STRING, &game_type_string,
    N_("Game choice: Regular or Colors"), N_("STRING")},
+  {"computer-test", 'c', 0, G_OPTION_ARG_INT, &test_computer_play,
+   N_("Number of computer-only games to play"), N_("NUMBER")},
+  {"monte-carlo-trials", 'm', 0, G_OPTION_ARG_INT, &NUM_TRIALS,
+   N_("Number of trials for each roll for the computer"), N_("NUMBER")},
   {NULL}
 };
 
@@ -207,7 +217,6 @@ CheerWinner (void)
     }
 
     update_score_state ();
-
   }
 
   if (players[winner].name)
@@ -251,6 +260,30 @@ do_computer_turns (void)
   return TRUE;
 }
 
+/* Show the current score and prompt for current player state */
+
+void
+DisplayCurrentPlayer() {
+  ShowoffPlayer (ScoreList, CurrentPlayer, 1);
+
+  if (players[CurrentPlayer].name) {
+    if (players[CurrentPlayer].comp) {
+      say (_("Computer playing for %s"), players[CurrentPlayer].name);
+    } else {
+      say (_("%s! -- You're up."), players[CurrentPlayer].name);
+    }
+  }
+}
+
+/* Display current player and refresh dice/display */
+void
+DisplayCurrentPlayerRefreshDice(void) {
+  DisplayCurrentPlayer();
+  UpdateAllDicePixmaps ();
+  DeselectAllDice ();
+  UpdateRollLabel();
+}
+
 void
 NextPlayer (void)
 {
@@ -259,7 +292,13 @@ NextPlayer (void)
       g_source_remove (last_timeout);
       last_timeout = 0;
     }
-    update_roll_button_sensitivity ();
+    if (DoDelay && NumberOfComputers > 0)
+        NumberOfRolls = NUM_ROLLS;
+    else
+        NumberOfRolls = LastHumanNumberOfRolls;
+    /* update_roll_button_sensitivity() needs to be called in
+       this context however UpdateRollLabel() also calls that method */
+    UpdateRollLabel ();
     CheerWinner ();
     return;
   }
@@ -273,18 +312,15 @@ NextPlayer (void)
     CurrentPlayer = (CurrentPlayer + 1) % NumberOfPlayers;
   } while (players[CurrentPlayer].finished);
 
-  ShowoffPlayer (ScoreList, CurrentPlayer, 1);
-
-  if (players[CurrentPlayer].name) {
-    if (players[CurrentPlayer].comp) {
-      say (_("Computer playing for %s"), players[CurrentPlayer].name);
-    } else {
-      say (_("%s! -- You're up."), players[CurrentPlayer].name);
-    }
-  }
-
+  DisplayCurrentPlayer();
   SelectAllDice ();
   RollSelectedDice ();
+  FreeRedoList();
+
+  /* Remember the roll count if this turn is for a
+     human player for display at the end of the game */
+  if (!players[CurrentPlayer].comp)
+    LastHumanNumberOfRolls = NumberOfRolls;
 
   if (players[CurrentPlayer].comp) {
     if (DoDelay) {
@@ -295,7 +331,51 @@ NextPlayer (void)
       do_computer_turns ();
     }
   }
-  UpdateRollLabel ();
+  /* Only update the roll label if we are running in
+    delay mode or if this turn is for a human player */
+  if (DoDelay || (!players[CurrentPlayer].comp))
+    UpdateRollLabel ();
+}
+
+/* Go back to the previous player */
+
+void
+PreviousPlayer(void)
+{
+  if (UndoPossible()) {
+    NumberOfRolls = 1;
+    ready_to_advance_player = FALSE;
+    ShowoffPlayer (ScoreList, CurrentPlayer, 0);
+
+    /* Find the next player with rolls left */
+    do {
+      CurrentPlayer = (UndoLastMove() + NumberOfPlayers) % NumberOfPlayers;
+    } while (players[CurrentPlayer].comp && UndoPossible());
+
+    DisplayCurrentPlayerRefreshDice();
+  }
+}
+
+void
+RedoPlayer(void)
+{
+  if (RedoPossible()) {
+    NumberOfRolls = 1;
+    ready_to_advance_player = FALSE;
+    ShowoffPlayer(ScoreList, CurrentPlayer, 0);
+    /* The first element of the list is the undone turn, so
+     * we need to remove it from the list before redoing other
+     * turns.                                                    */
+    FreeRedoListHead();
+
+    /* Redo all computer players */
+    do {
+      CurrentPlayer = RedoLastMove();
+    } while (players[CurrentPlayer].comp && RedoPossible());
+
+    RestoreLastRoll();
+    DisplayCurrentPlayerRefreshDice();
+  }
 }
 
 void
@@ -322,7 +402,7 @@ ShowPlayer (int num, int field)
       else
 	score = -1;
 
-      update_score_cell (ScoreList, line, num + 1, score);
+      if (test_computer_play == 0) update_score_cell (ScoreList, line, num + 1, score);
     }
   }
 
@@ -340,10 +420,12 @@ ShowPlayer (int num, int field)
     upper_tot += bonus;
   }
 
-  update_score_cell (ScoreList, R_BONUS, num + 1, bonus);
-  update_score_cell (ScoreList, R_UTOTAL, num + 1, upper_tot);
-  update_score_cell (ScoreList, R_LTOTAL, num + 1, lower_tot);
-  update_score_cell (ScoreList, R_GTOTAL, num + 1, upper_tot + lower_tot);
+  if (test_computer_play == 0) {
+      update_score_cell (ScoreList, R_BONUS, num + 1, bonus);
+      update_score_cell (ScoreList, R_UTOTAL, num + 1, upper_tot);
+      update_score_cell (ScoreList, R_LTOTAL, num + 1, lower_tot);
+      update_score_cell (ScoreList, R_GTOTAL, num + 1, upper_tot + lower_tot);
+  }
 }
 
 static gint
@@ -409,7 +491,9 @@ UpdateRollLabel (void)
   g_string_printf (str, "<b>%s %d/3</b>", _("Roll"), NumberOfRolls);
   gtk_label_set_label (GTK_LABEL (rollLabel), str->str);
 
+  update_score_tooltips ();
   update_roll_button_sensitivity ();
+  update_undo_sensitivity ();
 }
 
 static void
@@ -430,7 +514,7 @@ UpdateAllDicePixmaps (void)
   int i;
   static int prev_game_type = 0;
 
-  for (i = 0; i < NUMBER_OF_DICE; i++) {
+  for (i = 0; test_computer_play == 0 && i < NUMBER_OF_DICE; i++) {
     UpdateDiePixmap (i, prev_game_type);
   }
   prev_game_type = game_type;
@@ -441,6 +525,7 @@ DeselectAllDice (void)
 {
   int i;
 
+  if (test_computer_play > 0) return;
   for (i = 0; i < NUMBER_OF_DICE; i++)
     gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (diceBox[i]),
 				       FALSE);
@@ -481,7 +566,10 @@ gnome_roll_dice (GtkWidget * widget, GdkEvent * event, gpointer data)
 {
   if (!players[CurrentPlayer].comp) {
     RollSelectedDice ();
+    if (NumberOfRolls > 1)
+        FreeUndoRedoLists();
     UpdateRollLabel ();
+    LastHumanNumberOfRolls = NumberOfRolls;
   }
   return FALSE;
 }
@@ -493,6 +581,7 @@ say (char *fmt, ...)
   char buf[200];
   guint context_id;
 
+  if (test_computer_play > 0) return;
   va_start (ap, fmt);
   g_vsnprintf (buf, 200, fmt, ap);
   va_end (ap);
@@ -530,13 +619,15 @@ about_cb (GtkAction * action, gpointer data)
 			 "Free Software Foundation, Inc.",
 			 "license", license,
 			 "comments", _("A variation on poker with "
-				       "dice and less money."),
+				       "dice and less money.\n\n"
+				       "Tali is a part of GNOME Games."),
 			 "authors", authors,
 			 "documenters", documenters,
-			 "translator_credits", _("translator-credits"),
+			 "translator-credits", _("translator-credits"),
 			 "logo-icon-name", "gnome-tali",
 			 "website",
 			 "http://www.gnome.org/projects/gnome-games/",
+			 "website-label", _("GNOME Games web site"),
 			 "wrap-license", TRUE, NULL);
   g_free (license);
 
@@ -560,6 +651,12 @@ score_callback (GtkAction * action, gpointer data)
   return FALSE;
 }
 
+static gint undo_callback(GtkAction *action, gpointer data)
+{
+  PreviousPlayer();
+  return FALSE;
+}
+
 static void
 LoadDicePixmaps (void)
 {
@@ -571,8 +668,8 @@ LoadDicePixmaps (void)
      * code had a way for static images). */
     if (g_file_test (dicefiles[i], G_FILE_TEST_EXISTS))
       for (j = 0; j < NUMBER_OF_DICE; j++) {
-        dicePixmaps[j][i][GAME_YAHTZEE] = gtk_image_new_from_file(dicefiles[i]);
-        dicePixmaps[j][i][GAME_KISMET] = gtk_image_new_from_file(kdicefiles[i]);
+        dicePixmaps[j][i][GAME_YAHTZEE] = gtk_image_new_from_file (dicefiles[i]);
+        dicePixmaps[j][i][GAME_KISMET] = gtk_image_new_from_file (kdicefiles[i]);
       }
     /* FIXME: What happens if the file isn't found. */
   }
@@ -587,6 +684,12 @@ update_score_state (void)
   gtk_action_set_sensitive (scores_action, top != NULL);
 }
 
+void
+update_undo_sensitivity (void)
+{
+  gtk_action_set_sensitive(undo_action, UndoVisible());
+}
+
 static void
 help_cb (GtkAction * action, gpointer data)
 {
@@ -600,6 +703,8 @@ static const GtkActionEntry action_entry[] = {
   {"HelpMenu", NULL, N_("_Help")},
   {"NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL,
    G_CALLBACK (new_game_callback)},
+  {"Undo", GAMES_STOCK_UNDO_MOVE, NULL, NULL, NULL,
+   G_CALLBACK (undo_callback)},
   {"Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL,
    G_CALLBACK (score_callback)},
   {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_game)},
@@ -617,6 +722,7 @@ static const char ui_description[] =
   "  <menubar name='MainMenu'>"
   "    <menu action='GameMenu'>"
   "      <menuitem action='NewGame'/>"
+  "      <menuitem action='Undo'/>"
   "      <menuitem action='Scores'/>"
   "      <menuitem action='Quit'/>"
   "    </menu>"
@@ -643,6 +749,8 @@ create_menus (GtkUIManager * ui_manager)
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
   gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
   scores_action = gtk_action_group_get_action (action_group, "Scores");
+  undo_action   = gtk_action_group_get_action (action_group, "Undo");
+  update_undo_sensitivity();
 }
 
 
@@ -658,7 +766,10 @@ GyahtzeeCreateMainWindow (void)
   GtkUIManager *ui_manager;
   int i, j;
 
-  games_stock_init ();
+  window = gnome_app_new (appID, appName);
+
+  gtk_window_set_default_size (GTK_WINDOW (window), DEFAULT_WIDTH, DEFAULT_HEIGHT);
+  games_conf_add_window (GTK_WINDOW (window), NULL);
 
   g_signal_connect (G_OBJECT (window), "delete_event",
 		    G_CALLBACK (quit_game), NULL);
@@ -754,12 +865,14 @@ GyahtzeeCreateMainWindow (void)
 int
 main (int argc, char *argv[])
 {
-  GConfClient *client;
   GnomeProgram *program;
-  GSList *name_list = NULL;
-  gint i;
+  char **player_names;
+  gsize n_player_names;
+  guint i;
   GOptionContext *context;
 
+  g_thread_init (NULL);
+
   setgid_io_init ();
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -768,7 +881,7 @@ main (int argc, char *argv[])
   /* Reset all yahtzee variables before parsing args */
   YahtzeeInit ();
 
-  context = g_option_context_new ("");
+  context = g_option_context_new (NULL);
   g_option_context_add_main_entries (context, yahtzee_options,
 				     GETTEXT_PACKAGE);
 
@@ -779,20 +892,58 @@ main (int argc, char *argv[])
 				GNOME_PARAM_GOPTION_CONTEXT, context,
 				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
+  games_conf_initialise (appID);
+
+  games_stock_init ();
+
+  /* If we're in computer test mode, just run some tests, no GUI */
+  if (test_computer_play > 0) {
+      gint ii, jj, kk;
+      gdouble sum_scores = 0.0;
+      game_type = GAME_YAHTZEE;
+      if (game_type_string)
+          game_type = game_type_from_string(game_type_string);
+      g_message("In test computer play section - Using %d trials for simulation", NUM_TRIALS);
+      for (ii = 0; ii < test_computer_play; ii++) {
+          NumberOfHumans = 0;
+          NumberOfComputers = 1;
+          NewGame ();
+          int num_rolls = 0;
+
+          while (!GameIsOver() && num_rolls < 100) {
+              ComputerRolling (CurrentPlayer);
+              if (NoDiceSelected () || (NumberOfRolls >= NUM_ROLLS)) {
+                ComputerScoring (CurrentPlayer);
+                NumberOfRolls = 0;
+                SelectAllDice ();
+                RollSelectedDice ();
+              } else {
+                RollSelectedDice ();
+              }
+              num_rolls++;
+          }
+          for (kk = NumberOfHumans; kk < NumberOfPlayers; kk++) {
+              printf("Computer score: %d\n", total_score(kk));
+              sum_scores += total_score(kk);
+              if (num_rolls > 98) {
+                  for (jj = 0; jj < NUM_FIELDS; jj++)
+                      g_message("Category %d is score %d", jj, players[kk].score[jj]);
+              }
+          }
+      }
+      printf("Computer average: %.2f for %d trials\n", sum_scores / test_computer_play, NUM_TRIALS);
+      exit(0);
+  }
+
   highscores = games_scores_new (&scoredesc);
 
   gtk_window_set_default_icon_name ("gnome-tali");
 
-  client = gconf_client_get_default ();
   if (NumberOfComputers == 0)	/* Not set on the command-line. */
-    NumberOfComputers = gconf_client_get_int (client,
-					      "/apps/gtali/NumberOfComputerOpponents",
-					      NULL);
+    NumberOfComputers = games_conf_get_integer (NULL, KEY_NUMBER_OF_COMPUTERS, NULL);
 
   if (NumberOfHumans == 0)	/* Not set on the command-line. */
-    NumberOfHumans = gconf_client_get_int (client,
-					   "/apps/gtali/NumberOfHumanOpponents",
-					   NULL);
+    NumberOfHumans = games_conf_get_integer (NULL, KEY_NUMBER_OF_HUMANS, NULL);
 
   if (NumberOfHumans < 1)
     NumberOfHumans = 1;
@@ -804,50 +955,48 @@ main (int argc, char *argv[])
   if ((NumberOfHumans + NumberOfComputers) > MAX_NUMBER_OF_PLAYERS)
     NumberOfComputers = MAX_NUMBER_OF_PLAYERS - NumberOfHumans;
 
-  if (game_type_string) 
+  if (game_type_string)
     game_type = game_type_from_string(game_type_string);
-  else
-    game_type = game_type_from_string(gconf_client_get_string(client,
-                                      "/apps/gtali/GameType", NULL));
+  else {
+    char *type;
+
+    type = games_conf_get_string (NULL, KEY_GAME_TYPE, NULL);
+    game_type = game_type_from_string(type);
+  }
+
   set_new_game_type(game_type);
 
-  /* We ignore errors for these, they're boolean so it will be valid 
-   * data even if not the right data and there's nothing else we
-   * can do (if it's a systematic gconf problem, the calls above
-   * would have caught it. */
+  if (NUM_TRIALS <= 0)
+      NUM_TRIALS = games_conf_get_integer (NULL, KEY_NUMTRIALS, NULL);
+
   if (DoDelay == 0)		/* Not set on the command-line */
-    DoDelay = gconf_client_get_bool (client,
-				     "/apps/gtali/DelayBetweenRolls", NULL);
+    DoDelay = games_conf_get_boolean (NULL, KEY_DELAY_BETWEEN_ROLLS, NULL);
   if (DisplayComputerThoughts == 0)	/* Not set on the command-line */
-    DisplayComputerThoughts = gconf_client_get_bool (client,
-						     "/apps/gtali/DisplayComputerThoughts",
-						     NULL);
+    DisplayComputerThoughts = games_conf_get_boolean (NULL, KEY_DISPLAY_COMPUTER_THOUGHTS, NULL);
+  
   /* Read in new player names */
-  name_list = gconf_client_get_list (client, "/apps/gtali/PlayerNames",
-				     GCONF_VALUE_STRING, NULL);
+  player_names = games_conf_get_string_list (NULL, KEY_PLAYER_NAMES, &n_player_names, NULL);
+  if (player_names) {
+    n_player_names = MIN (n_player_names, MAX_NUMBER_OF_PLAYERS);
 
-  for (i = 0; i < MAX_NUMBER_OF_PLAYERS && name_list; i++) {
-    if (name_list->data) {
-      if (!i && strcasecmp (name_list->data, _("Human")) == 0) {
-	char *realname;
+    for (i = 0; i < n_player_names; ++i) {
+      if (i == 0 && strcasecmp (player_names[i], _("Human")) == 0) {
+	const char *realname;
 
-	realname = g_strndup (g_get_real_name (), 255);
-	if (strcasecmp (realname, "Unknown") == 0 || strlen (realname) < 1) {
-	  g_free (realname);
-	  realname = g_strdup (name_list->data);
+	realname = g_get_real_name ();
+        if (realname && realname[0] && strcmp (realname, "Unknown") != 0) {
+          players[i].name = g_locale_to_utf8 (realname, -1, NULL, NULL, NULL);
+        }
+        if (!players[i].name) {
+	  players[i].name = g_strdup (player_names[i]);
 	}
-	g_strdelimit (realname, " ", '\0');
-	players[i].name = realname;
       } else {
-	players[i].name = g_strdup (name_list->data);
+        players[i].name = g_strdup (player_names[i]);
       }
     }
 
-    name_list = g_slist_next (name_list);
+    g_strfreev (player_names);
   }
-  g_slist_free (name_list);
-
-  window = gnome_app_new (appID, appName);
 
   GyahtzeeCreateMainWindow ();
 
@@ -856,11 +1005,11 @@ main (int argc, char *argv[])
 
   gtk_main ();
 
-  gnome_accelerators_sync ();
+  games_conf_shutdown ();
 
   g_object_unref (program);
 
-  return 0;
+  exit(0);
 }
 
 /* Arrgh - lets all use the same tabs under emacs: 
